<!DOCTYPE html>
<html lang="en">

<head>
    {% block head %}
    <style type="text/css">
    {% include "style.css" %}
    {{ custom_css }}
    </style>
    <script>
    {% include "polyfill.js" %}
    </script>
    <script id="MathJax-script">
    window.MathJax = {
        tex: {
            inlineMath: [['$', '$'], ['\\(', '\\)']]
        }
    };
    {% include "mathjax.js" %}
    </script>
    <meta charset="utf-8" />
    <title>{% block title %}{% endblock %} - SWIFT Pipeline</title>
    {% endblock %}
</head>

<body>
    <div id="navigation">{% block navigation %}{% endblock %}</div>

    <div id="content">{% block content %}{% endblock %}</div>

    <div id="footer">
        {% block footer %}{% endblock %}
    </div>
</body>

</html>